library(tidyverse)

args     <- commandArgs(trailingOnly = TRUE)

data_dir <- args[[1]]
m        <- as.numeric(args[[2]])
k        <- as.numeric(args[[3]])
seed     <- as.integer(args[[4]])

set.seed(seed)

df_gose <- readRDS(sprintf("%s/df_gose.rds", data_dir))

# get ids of all subjects with a valid 6mo GOSE
test_candidate_ids <- df_gose %>%
  group_by(gupi) %>%
  filter(
    # at least one non-death observation close to 6 months
    any(
      (Outcomes.DerivedCompositeGOSEDaysPostInjury >= 180 - 14) &
      (Outcomes.DerivedCompositeGOSE != 1) &
      (Outcomes.DerivedCompositeGOSEDaysPostInjury <= 180 + 14)
    ),
    # at least one observation not close to 6 months
    any(
      (Outcomes.DerivedCompositeGOSEDaysPostInjury < 180 - 14) |
      (Outcomes.DerivedCompositeGOSEDaysPostInjury > 180 + 14)
    )
  ) %>%
  .[["gupi"]] %>%
  unique %>%
  as.character

df_validation <- tibble(
  gupi       = df_gose$gupi %>% unique,
  valid_test = gupi %in% test_candidate_ids
) %>%
  group_by(
    valid_test
  ) %>%
  mutate(
    fold = sample(rep(1:k, ceiling(n()/k)))[1:n()]
  ) %>%
  ungroup

# saveRDS(df_validation, "df_validation_splits.rds")

for (i in 1:m) {

  df_baseline <- readRDS(sprintf("%s/mi_baseline/df_baseline_mi_%i.rds", data_dir, i))

  for (j in 1:k) {

    df_test_rows <- df_gose %>%
      left_join(df_baseline, by = "gupi") %>%
      left_join(df_validation, by = "gupi") %>%
      filter(fold == j & valid_test) %>%
      group_by(gupi) %>%
      filter(
        Outcomes.DerivedCompositeGOSEDaysPostInjury ==
          Outcomes.DerivedCompositeGOSEDaysPostInjury[which.min(abs(Outcomes.DerivedCompositeGOSEDaysPostInjury - 180))]) %>%
      ungroup() %>%
      select(
        gupi, Outcomes.DerivedCompositeGOSEDaysPostInjury
      ) %>%
      mutate(test = TRUE) %>%
      right_join(
        df_gose %>% select(gupi, Outcomes.DerivedCompositeGOSEDaysPostInjury),
        by = c("gupi", "Outcomes.DerivedCompositeGOSEDaysPostInjury")
      ) %>%
      mutate(test = ifelse(is.na(test), FALSE, test))

    df_test <- df_gose %>%
      left_join(df_test_rows, by = c("gupi", "Outcomes.DerivedCompositeGOSEDaysPostInjury")) %>%
      filter(test) %>%
      select(-test) %>%
      left_join(df_baseline, by = "gupi")

    df_train <- df_gose %>%
      left_join(df_test_rows, by = c("gupi", "Outcomes.DerivedCompositeGOSEDaysPostInjury")) %>%
      filter(!test) %>%
      select(-test) %>%
      left_join(df_baseline, by = "gupi")

    # plausi checks
    if (nrow(df_test) + nrow(df_train) != nrow(df_gose)) stop("test/train does not add up")
    if (any(df_test$Outcomes.DerivedCompositeGOSE == 1)) stop("test cases cannot be GOSE 1")

    saveRDS(df_test, sprintf("%s/validation/df_test_mi_%i_fold_%i.rds", data_dir, i, j))
    saveRDS(df_train, sprintf("%s/validation/df_train_mi_%i_fold_%i.rds", data_dir, i, j))

  }
}
